<?php


namespace Drupal\views_access_user\Plugin\views\access;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsAccess;
use Drupal\views\Plugin\views\access\AccessPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Access plugin that provides access on a per-user basis.
 *
 * @ingroup views_access_plugins
 */
#[ViewsAccess(
    id: 'user',
    title: new TranslatableMarkup('User'),
    help: new TranslatableMarkup('Access will be granted to individual users.'),
)]
class User extends AccessPluginBase implements CacheableDependencyInterface
{

    protected $usesOptions = TRUE;

    protected $entityTypeManager;

    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
    {
        $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
        $instance->entityTypeManager = $container->get('entity_type.manager');
        return $instance;
    }

    public function summaryTitle() {
        $uids = is_array($this->options['users']) ? $this->options['users'] : [];
        $users = $uids ? $this->entityTypeManager->getStorage('user')->loadMultiple($uids) : [];
        $names = array_map(static fn($user) => $user->getDisplayName(), $users);

        $output = implode(', ', $names);
        $max_length = 35;
        if (strlen($output) > $max_length) {
            $output = mb_substr($output, 0, $max_length) . '...';
        }

//        $icon = fn(bool $value) => $value ? '✔️' : '❌';
        $icon = fn(bool $value) => $value ? 'Yes' : 'No';

        return $this->t('Anon: @anon • Users: @users', [
            '@anon' => $icon(!empty($this->options['allow_anonymous'])),
            '@users' => $output,
        ]);
    }

    public function access(AccountInterface $account)
    {
        $route_name = \Drupal::routeMatch()->getRouteName();
        if ($account->hasPermission('administer views') && str_starts_with($route_name, 'views_ui.')) {
            return TRUE;
        }

        if ($account->isAnonymous() && !empty($this->options['allow_anonymous'])) {
            return TRUE;
        }

        return in_array($account->id(), $this->options['users'], TRUE);
    }

    public function alterRouteDefinition(Route $route)
    {
        $route->setRequirement('_user_is_logged_in', TRUE);
    }

    protected function defineOptions()
    {
        $options = parent::defineOptions();
        $options['users'] = ['default' => []];
        $options['allow_anonymous'] = ['default' => FALSE];
        return $options;
    }

    public function buildOptionsForm(&$form, FormStateInterface $form_state) {
        parent::buildOptionsForm($form, $form_state);

        $form['allow_anonymous'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Allow Anonymous Users'),
            '#default_value' => $this->options['allow_anonymous'],
            '#description' => $this->t('If checked, users who are not logged in will also be able to access this view.'),
        ];

        $user_storage = $this->entityTypeManager->getStorage('user');
        $uids = is_array($this->options['users']) ? $this->options['users'] : [];
        $users = $user_storage->loadMultiple($uids);

        $default_lines = implode("\n", array_map(
            fn($user) => $user->getAccountName(),
            $users
        ));

        $form['users'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Allowed Users'),
            '#default_value' => $default_lines,
            '#description' => $this->t('Enter one username per line. Only letters, numbers, periods, underscores, or hyphens are allowed. Duplicate usernames not allowed.'),
            '#attributes' => [
                'style' => 'width: 100%; min-height: 25em; max-height: 50em; resize: vertical;',
            ],
        ];
    }

    public function validateOptionsForm(&$form, FormStateInterface $form_state) {
        $input = $form_state->getValue(['access_options', 'users']) ?? '';
        $lines = array_map('trim', explode("\n", $input));
        $lines = array_filter($lines);

        $uids = [];
        $seen = [];
        $user_storage = $this->entityTypeManager->getStorage('user');

        foreach ($lines as $username) {
            // Validate allowed characters only
            if (!preg_match('/^[A-Za-z0-9._-]+$/', $username)) {
                $form_state->setError($form['users'], $this->t('Invalid characters in username "@name".', ['@name' => $username]));
                continue;
            }

            // Check for duplicates
            if (in_array(strtolower($username), $seen)) {
                $form_state->setError($form['users'], $this->t('Duplicate username "@name" found.', ['@name' => $username]));
                continue;
            }
            $seen[] = strtolower($username);

            // User lookup
            $users = $user_storage->loadByProperties(['name' => $username]);
            $user = reset($users);
            if ($user) {
                $uids[] = $user->id();
            } else {
                $form_state->setError($form['users'], $this->t('User "@name" does not exist. Make sure that the username is correct and try again.', ['@name' => $username]));
            }
        }

        if (!$form_state->getErrors()) {
            $form_state->setValue(['access_options', 'users'], $uids);
        }
    }

    public function submitOptionsForm(&$form, FormStateInterface $form_state) {
        $this->options['users'] = $form_state->getValue(['access_options', 'users']) ?? [];
        $this->options['allow_anonymous'] = (bool) $form_state->getValue(['access_options', 'allow_anonymous']);
    }

    public function getCacheMaxAge()
    {
        return Cache::PERMANENT;
    }

    public function getCacheContexts()
    {
        return ['user'];
    }

    public function getCacheTags()
    {
        return [];
    }

}